/* eslint-disable no-magic-numbers */
import {
  allBioEntitesSelectorOfCurrentMap,
  allSubmapConnectionsBioEntityOfCurrentSubmapWithRealConnectionsSelector,
  allVisibleBioEntitiesIdsSelector,
} from '@/redux/bioEntity/bioEntity.selectors';
import { allChemicalsBioEntitesOfCurrentMapSelector } from '@/redux/chemicals/chemicals.selectors';
import { allDrugsBioEntitesOfCurrentMapSelector } from '@/redux/drugs/drugs.selectors';
import { entityNumberDataSelector } from '@/redux/entityNumber/entityNumber.selectors';
import { markersPinsOfCurrentMapDataSelector } from '@/redux/markers/markers.selectors';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import Feature from 'ol/Feature';
import { Geometry } from 'ol/geom';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useMemo } from 'react';
import { useSelector } from 'react-redux';
import { getBioEntitiesFeatures } from './getBioEntitiesFeatures';
import { getMarkersFeatures } from './getMarkersFeatures';

export const useOlMapPinsLayer = (): VectorLayer<VectorSource<Feature<Geometry>>> => {
  const pointToProjection = usePointToProjection();
  const activeIds = useSelector(allVisibleBioEntitiesIdsSelector);
  const contentBioEntites = useSelector(allBioEntitesSelectorOfCurrentMap);
  const chemicalsBioEntities = useSelector(allChemicalsBioEntitesOfCurrentMapSelector);
  const drugsBioEntities = useSelector(allDrugsBioEntitesOfCurrentMapSelector);
  const markersEntities = useSelector(markersPinsOfCurrentMapDataSelector);
  const entityNumber = useSelector(entityNumberDataSelector);
  const submapConnections = useSelector(
    allSubmapConnectionsBioEntityOfCurrentSubmapWithRealConnectionsSelector,
  );

  const elementsFeatures = useMemo(
    () =>
      [
        getBioEntitiesFeatures(contentBioEntites, {
          pointToProjection,
          type: 'bioEntity',
          entityNumber,
          activeIds,
        }),
        getBioEntitiesFeatures(chemicalsBioEntities, {
          pointToProjection,
          type: 'chemicals',
          entityNumber,
          activeIds,
        }),
        getBioEntitiesFeatures(drugsBioEntities, {
          pointToProjection,
          type: 'drugs',
          entityNumber,
          activeIds,
        }),
        getBioEntitiesFeatures(submapConnections, {
          pointToProjection,
          type: 'bioEntity',
          entityNumber,
        }),
        getMarkersFeatures(markersEntities, { pointToProjection }),
      ].flat(),
    [
      contentBioEntites,
      drugsBioEntities,
      chemicalsBioEntities,
      pointToProjection,
      markersEntities,
      entityNumber,
      activeIds,
      submapConnections,
    ],
  );

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features: [...elementsFeatures],
    });
  }, [elementsFeatures]);

  const pinsLayer = useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );

  return pinsLayer;
};
